@@ -20,124 +20,200 @@
 
 #include "common.h"
 
-#ifdef USE_WIN32
-#define Win32_Winsock
-#include <windows.h>
-#endif
-
-#include <stdio.h>
-#include <stdarg.h>    /* for va_* */
-#include <unistd.h>    /* for read(), write() */
-#include <string.h>
-#include <sys/time.h>  /* for select() */
-#include <sys/types.h> /* Ultrix needs it for fd_set */
-
 /* protocol-specific function prototypes */
-static int smb_client(int, int);
-static int smb_server(int, int);
-static int smtp_client(int, int);
-static int smtp_server(int, int);
-static int telnet_client(int, int);
-static int telnet_server(int, int);
+static int smb_client(int, int, int);
+static int smb_server(int, int, int);
+static int smtp_client(int, int, int);
+static int smtp_server(int, int, int);
+static int pop3_client(int, int, int);
+static int pop3_server(int, int, int);
+static int nntp_client(int, int, int);
+static int nntp_server(int, int, int);
+static int telnet_client(int, int, int);
+static int telnet_server(int, int, int);
 static int RFC2487(int);
 
 /* descriptor versions of fprintf/fscanf */
 static int fdprintf(int, char *, ...);
 static int fdscanf(int, char *, char *);
 
-int negotiate(char *protocol, int client, int local, int remote)
-{
+int negotiate(char *protocol, int client, int local_rd, int local_wr, int remote) {
     if(!protocol)
         return 0; /* No protocol negotiations */
     log(LOG_DEBUG, "Negotiations for %s(%s side) started", protocol,
         client ? "client" : "server");
     if(!strcmp(protocol, "smb")) {
         if(client)
-            return smb_client(local, remote);
+            return smb_client(local_rd, local_wr, remote);
         else
-            return smb_server(local, remote);
+            return smb_server(local_rd, local_wr, remote);
     }
     if(!strcmp(protocol, "smtp")) {
         if(client)
-            return smtp_client(local, remote);
+            return smtp_client(local_rd, local_wr, remote);
         else
-            return smtp_server(local, remote);
+            return smtp_server(local_rd, local_wr, remote);
+    }
+    if(!strcmp(protocol, "pop3")) {
+        if(client)
+            return pop3_client(local_rd, local_wr, remote);
+        else
+            return pop3_server(local_rd, local_wr, remote);
+    }
+    if(!strcmp(protocol, "nntp")) {
+        if(client)
+            return nntp_client(local_rd, local_wr, remote);
+        else
+            return nntp_server(local_rd, local_wr, remote);
     }
     if(!strcmp(protocol, "telnet")) {
         if(client)
-            return telnet_client(local, remote);
+            return telnet_client(local_rd, local_wr, remote);
         else
-            return telnet_server(local, remote);
+            return telnet_server(local_rd, local_wr, remote);
     }
     log(LOG_ERR, "Protocol %s not supported in %s mode",
         protocol, client ? "client" : "server");
     return -1;
 }
 
-static int smb_client(int local, int remote)
-{
+static int smb_client(int local_rd, int local_wr, int remote) {
     log(LOG_ERR, "Protocol not supported");
     return -1;
 }
 
-static int smb_server(int local, int remote)
-{
+static int smb_server(int local_rd, int local_wr, int remote) {
     log(LOG_ERR, "Protocol not supported");
     return -1;
 }
 
-static int smtp_client(int local, int remote)
-{
-    log(LOG_ERR, "Protocol not supported");
-    return -1;
+static int smtp_client(int local_rd, int local_wr, int remote) {
+    char line[STRLEN];
+    
+    do { /* Copy multiline greeting */
+        if(fdscanf(remote, "%[^\n]", line)<0)
+            return -1;
+        if(fdprintf(local_wr, line)<0)
+            return -1;
+    } while(strncmp(line,"220-",4)==0);
+
+    if(fdprintf(remote, "EHLO localhost")<0) /* Send an EHLO command */
+        return -1;
+    do { /* Skip multiline reply */
+        if(fdscanf(remote, "%[^\n]", line)<0)
+            return -1;
+    } while(strncmp(line,"250-",4)==0);
+    if(strncmp(line,"250 ",4)!=0) { /* Error */
+        log(LOG_ERR, "Remote server is not RFC 1425 compliant");
+        return -1;
+    }
+
+    if(fdprintf(remote, "STARTTLS")<0) /* Send STARTTLS command */
+        return -1;
+    do { /* Skip multiline reply */
+        if(fdscanf(remote, "%[^\n]", line)<0)
+            return -1;
+    } while(strncmp(line,"220-",4)==0);
+    if(strncmp(line,"220 ",4)!=0) { /* Error */
+        log(LOG_ERR, "Remote server is not RFC 2487 compliant");
+        return -1;
+    }
+    return 0;
 }
 
-static int smtp_server(int local, int remote)
-{
+static int smtp_server(int local_rd, int local_wr, int remote) {
     char line[STRLEN];
 
-    if(RFC2487(local)==0)
+    if(RFC2487(local_rd)==0)
         return 0; /* Return if RFC 2487 is not used */
 
     if(fdscanf(remote, "220%[^\n]", line)!=1) {
         log(LOG_ERR, "Unknown server welcome");
         return -1;
     }
-    if(fdprintf(local, "220%s + stunnel", line)<0)
+    if(fdprintf(local_wr, "220%s + stunnel", line)<0)
         return -1;
-    if(fdscanf(local, "EHLO %[^\n]", line)!=1) {
+    if(fdscanf(local_rd, "EHLO %[^\n]", line)!=1) {
         log(LOG_ERR, "Unknown client EHLO");
         return -1;
     }
-    if(fdprintf(local, "250-%s Welcome", line)<0)
+    if(fdprintf(local_wr, "250-%s Welcome", line)<0)
         return -1;
-    if(fdprintf(local, "250 STARTTLS")<0)
+    if(fdprintf(local_wr, "250 STARTTLS")<0)
         return -1;
-    if(fdscanf(local, "STARTTLS", line)<0) {
+    if(fdscanf(local_rd, "STARTTLS", line)<0) {
         log(LOG_ERR, "STARTTLS expected");
         return -1;
     }
-    if(fdprintf(local, "220 Go ahead", line)<0)
+    if(fdprintf(local_wr, "220 Go ahead", line)<0)
         return -1;
     return 0;
 }
 
-static int telnet_client(int local, int remote)
-{
+static int pop3_client(int local_rd, int local_wr, int remote) {
+    char line[STRLEN];
+
+    fdscanf(remote, "%[^\n]", line);
+    if(strncmp(line,"+OK ",4)) {
+        log(LOG_ERR, "Unknown server welcome");
+        return -1;
+    }
+    if(fdprintf(local_wr, line)<0)
+        return -1;
+    if(fdprintf(remote, "STLS")<0)
+        return -1;
+    fdscanf(remote, "%[^\n]", line);
+    if(strncmp(line,"+OK ",4)) {
+        log(LOG_ERR, "Server does not support TLS");
+        return -1;
+    }
+    return 0;
+}
+
+static int pop3_server(int local_rd, int local_wr, int remote) {
+    log(LOG_ERR, "Protocol not supported in server mode");
+    return -1;
+}
+
+static int nntp_client(int local_rd, int local_wr, int remote) {
+    char line[STRLEN];
+
+    fdscanf(remote, "%[^\n]", line);
+    if(strncmp(line,"200 ",4) && strncmp(line,"201 ",4)) {
+        log(LOG_ERR, "Unknown server welcome");
+        return -1;
+    }
+    if(fdprintf(local_wr, line)<0)
+        return -1;
+    if(fdprintf(remote, "STARTTLS")<0)
+        return -1;
+    fdscanf(remote, "%[^\n]", line);
+    if(strncmp(line,"382 ",4)) {
+        log(LOG_ERR, "Server does not support TLS");
+        return -1;
+    }
+    return 0;
+}
+
+static int nntp_server(int local_rd, int local_wr, int remote) {
+    log(LOG_ERR, "Protocol not supported in server mode");
+    return -1;
+}
+
+static int telnet_client(int local_rd, int local_wr, int remote) {
     log(LOG_ERR, "Protocol not supported");
     return -1;
 }
 
-static int telnet_server(int local, int remote)
-{
+static int telnet_server(int local_rd, int local_wr, int remote) {
     log(LOG_ERR, "Protocol not supported");
     return -1;
 }
 
-static int fdprintf(int fd, char *format, ...)
-{
+static int fdprintf(int fd, char *format, ...) {
     va_list arglist;
-    char line[STRLEN], *crlf="\r\n";
+    char line[STRLEN], logline[STRLEN];
+    char *crlf="\r\n";
     int len;
 
     va_start(arglist, format);
@@ -155,13 +231,14 @@ static int fdprintf(int fd, char *format, ...)
         sockerror("writesocket (fdprintf)");
         return -1;
     }
+    safecopy(logline, line);
+    safestring(logline);
     log(LOG_DEBUG, " -> %s", line);
     return len;
 }
 
-static int fdscanf(int fd, char *format, char *buffer)
-{
-    char line[STRLEN];
+static int fdscanf(int fd, char *format, char *buffer) {
+    char line[STRLEN], logline[STRLEN];
     int ptr;
 
     ptr=0;
@@ -182,7 +259,9 @@ static int fdscanf(int fd, char *format, char *buffer)
             break;
     }
     line[ptr]='\0';
-    log(LOG_DEBUG, " <- %s", line);
+    safecopy(logline, line);
+    safestring(logline);
+    log(LOG_DEBUG, " <- %s", logline);
     return sscanf(line, format, buffer);
 }
 
@@ -197,8 +276,7 @@ static int fdscanf(int fd, char *format, char *buffer)
 *
 */
 
-static int RFC2487(int fd)
-{
+static int RFC2487(int fd) {
     fd_set         fdsRead;
     struct timeval timeout;
 
@@ -218,3 +296,4 @@ static int RFC2487(int fd)
     return -1;
 }
 
+/* End of protocol.c */
