@@ -19,7 +19,6 @@
  */
 
 #include "common.h"
-#include <stdio.h>
 
 #ifdef HAVE_OPENSSL
 #include <openssl/crypto.h> /* for CRYPTO_* */
@@ -27,24 +26,20 @@
 #include <crypto.h> /* for CRYPTO_* */
 #endif
 
-#define MAX_CRIT_SECTIONS 3
-
 #ifdef USE_PTHREAD
 
 #include <pthread.h>
-#include <unistd.h> /* for getpid() */
-#include <signal.h> /* for sigemptyset(), sigaddset() */
 
-pthread_mutex_t stunnel_cs[MAX_CRIT_SECTIONS];
+pthread_mutex_t stunnel_cs[CRIT_SECTIONS];
 
 pthread_mutex_t lock_cs[CRYPTO_NUM_LOCKS];
 pthread_attr_t pth_attr;
 
-void enter_critical_section(int i) {
+void enter_critical_section(section_code i) {
     pthread_mutex_lock(stunnel_cs+i);
 }
 
-void leave_critical_section(int i) {
+void leave_critical_section(section_code i) {
     pthread_mutex_unlock(stunnel_cs+i);
 }
 
@@ -52,8 +47,7 @@ static void locking_callback(int mode, int type,
 #ifdef HAVE_OPENSSL
     const /* Callback definition has been changed in openssl 0.9.3 */
 #endif
-    char *file, int line)
-{
+    char *file, int line) {
     if(mode&CRYPTO_LOCK)
         pthread_mutex_lock(lock_cs+type);
     else
@@ -64,7 +58,7 @@ void sthreads_init() {
     int i;
 
     /* Initialize stunnel critical sections */
-    for(i=0; i<MAX_CRIT_SECTIONS; i++)
+    for(i=0; i<CRIT_SECTIONS; i++)
         pthread_mutex_init(stunnel_cs+i, NULL);
 
     /* Initialize OpenSSL locking callback */
@@ -91,6 +85,10 @@ int create_client(int ls, int s, void (*cli)(int)) {
 
      sigemptyset(&mask);
      sigaddset(&mask, SIGCHLD);
+     sigaddset(&mask, SIGTERM);
+     sigaddset(&mask, SIGQUIT);
+     sigaddset(&mask, SIGINT);
+     sigaddset(&mask, SIGHUP);
      pthread_sigmask(SIG_BLOCK, &mask, &oldmask); /* block SIGCHLD */
      if(pthread_create(&thread, &pth_attr, (void *)cli, (void *)s)) {
          /* SIGCHLD will remain blocked here */
@@ -105,15 +103,13 @@ int create_client(int ls, int s, void (*cli)(int)) {
 
 #ifdef USE_WIN32
 
-#include <windows.h>
+CRITICAL_SECTION stunnel_cs[CRIT_SECTIONS];
 
-CRITICAL_SECTION stunnel_cs[MAX_CRIT_SECTIONS];
-
-void enter_critical_section(int i) {
+void enter_critical_section(section_code i) {
     EnterCriticalSection(stunnel_cs+i);
 }
 
-void leave_critical_section(int i) {
+void leave_critical_section(section_code i) {
     LeaveCriticalSection(stunnel_cs+i);
 }
 
@@ -121,7 +117,7 @@ void sthreads_init() {
     int i;
 
     /* Initialize stunnel critical sections */
-    for(i=0; i<MAX_CRIT_SECTIONS; i++)
+    for(i=0; i<CRIT_SECTIONS; i++)
         InitializeCriticalSection(stunnel_cs+i);
 }
 
@@ -145,14 +141,11 @@ int create_client(int ls, int s, void (*cli)(int)) {
 
 #ifdef USE_FORK
 
-#include <unistd.h> /* for getpid() */
-#include <signal.h> /* for signal() */
-
-void enter_critical_section(int i) {
+void enter_critical_section(section_code i) {
     /* empty */
 }
 
-void leave_critical_section(int i) {
+void leave_critical_section(section_code i) {
     /* empty */
 }
 
@@ -185,3 +178,5 @@ int create_client(int ls, int s, void (*cli)(int)) {
 }
 
 #endif
+
+/* End of sthreads.c */
